@@ -6,7 +6,6 @@ from inspect import isgeneratorfunction
 from itertools import chain, groupby
 from pathlib import PurePath
 
-
 @contextmanager
 def _connect(db):
     conn = sqlite3.connect(db)
@@ -41,12 +40,12 @@ def _delayed_keyboard_interrupts():
             old_handler(*signal_received)
 
 
-class Inst:
+class _Instruction:
     def __init__(self, iterable):
         def gen():
             yield from iterable
 
-        self._iter_stack = [iterable if isgeneratorfunction(iterable) else gen]
+        self._iter_stack = [_it2gen(iterable)]
         self._is_grouped = False
 
     def map(self, fn):
@@ -89,6 +88,7 @@ class Inst:
         fn = _fn2gen(fn)
 
         empty = object()
+
         def _step(krs1, krs2):
             try:
                 k1, rs1 = next(krs1)
@@ -96,16 +96,16 @@ class Inst:
                 while True:
                     if k1 == k2:
                         yield from fn(list(rs1), list(rs2))
-                        k1 = k2 = empty 
+                        k1 = k2 = empty
                         k1, rs1 = next(krs1)
                         k2, rs2 = next(krs2)
                     elif k1 < k2:
                         yield from fn(list(rs1), [])
-                        k1 = empty 
+                        k1 = empty
                         k1, rs1 = next(krs1)
                     else:
                         yield from fn([], list(rs2))
-                        k2 = empty 
+                        k2 = empty
                         k2, rs2 = next(krs2)
             except StopIteration:
                 # unconsumed
@@ -118,9 +118,10 @@ class Inst:
                     yield from fn(list(rs1), [])
                 for _, rs2 in krs2:
                     yield from fn([], list(rs2))
-        
+
         selfpg = self._iter_stack[-1]
-        otherpg = other._iter_stack[-1]
+        otherpg = other._iter_stack[-1] if isinstance(other, _Instruction)\
+            else _it2gen(other) 
 
         def gen():
             yield from _step(selfpg(), otherpg())
@@ -132,7 +133,8 @@ class Inst:
     def concat(self, other):
 
         selfpg = self._iter_stack[-1]
-        otherpg = other._iter_stack[-1]
+        otherpg = other._iter_stack[-1] if isinstance(other, _Instruction)\
+            else _it2gen(other)
 
         def gen():
             for r in selfpg():
@@ -143,6 +145,12 @@ class Inst:
         self._iter_stack.append(gen)
         return self
 
+    def iter(self):
+        yield from self._iter_stack[-1]()
+
+    def list(self):
+        return list(self.iter())
+
 
 class Conn:
     def __init__(self, dbfile):
@@ -154,29 +162,31 @@ class Conn:
 
         self._conn = None
 
-    def inst(self, tname, by=None):
+    def __getitem__(self, tname):
+        bycols = None
+        if isinstance(tname, tuple):
+            tname, bycols = tname
+
         def gen():
-            yield from _fetch(self._conn, tname, by)
+            try:
+                yield from _fetch(self._conn, tname, bycols)
+            # in case self._conn is either NoneType or closed connection
+            except (AttributeError, sqlite3.ProgrammingError):
+                with _connect(self._db) as c:
+                    yield from _fetch(c, tname, bycols)
 
-        nst = Inst(gen)
-        if by:
-            nst._is_grouped = True
+        inst = _Instruction(gen)
+        if bycols:
+            inst._is_grouped = True
 
-        return nst
+        return inst
 
-    def __setitem__(self, tname, nst):
+    def __setitem__(self, tname, val):
         with _connect(self._db) as c:
             self._conn = c
             _delete(c, tname)
-            _insert(c, tname, nst._iter_stack[-1]())
-
-    def __getitem__(self, tname):
-        try:
-            return list(_fetch(self._conn, tname, None))
-        # in case self._conn is either NoneType or closed connection
-        except (AttributeError, sqlite3.ProgrammingError):
-            with _connect(self._db) as c:
-                return list(_fetch(c, tname, None))
+            itr = val.iter() if isinstance(val, _Instruction) else iter(val)
+            _insert(c, tname, itr) 
 
 
 def _insert_statement(name, d):
@@ -230,6 +240,7 @@ def _delete(c, tname):
 
 
 def _insert(c, tname, rs):
+    rs = iter(rs)
     try:
         r0 = next(rs)
     except StopIteration:
@@ -252,6 +263,7 @@ def _fetch(c, tname, cols=None):
         yield from c.cursor().execute(query)
 
 
+# ordinary function to generator function
 def _fn2gen(f):
     def gen(*r):
         x = f(*r)
@@ -261,3 +273,11 @@ def _fn2gen(f):
             yield from x
 
     return f if isgeneratorfunction(f) else gen
+
+
+# iterable to generator function with no argument
+def _it2gen(iterable):
+    def gen():
+        yield from iterable
+    return iterable if isgeneratorfunction(iterable) else gen
+
