@@ -1,8 +1,8 @@
 import unittest
 import os
 from pathlib import Path
-
-from tablemap import Conn, Inst
+from itertools import groupby
+from tablemap import Conn
 
 
 table1 = [
@@ -41,44 +41,48 @@ class TestGeneral(unittest.TestCase):
             yield r
 
         conn = Conn(self.dbfile)
-        conn['t1'] = Inst(table1)
-        conn['t1_sum'] = conn.inst('t1', by='col1')\
+        conn['t1'] = table1
+        conn['t1_sum'] = conn['t1', 'col1']\
             .map(sum_by_col1)
 
-        self.assertEqual(conn['t1_sum'],
-                         [{'col1': 'a', 'col2': 9}, {'col1': 'b', 'col2': 1}, {
-                             'col1': 'c', 'col2': 14}, {'col1': 'd', 'col2': 2}]
+        self.assertEqual(list(conn['t1_sum'].iter()),
+                         [{'col1': 'a', 'col2': 9},
+                          {'col1': 'b', 'col2': 1},
+                          {'col1': 'c', 'col2': 14},
+                          {'col1': 'd', 'col2': 2}]
                          )
-        conn['t1_sum2'] = conn.inst('t1')\
+        conn['t1_sum2'] = conn['t1']\
             .by('col1').map(sum_by_col1)
-        
-        self.assertEqual(conn['t1_sum'], conn['t1_sum2'])
+
+        self.assertEqual(conn['t1_sum'].list(), conn['t1_sum2'].list())
 
     def test_full_join(self):
         def add_col3(rs1, rs2):
+            # inner join
             if rs1 and rs2:
                 for r1 in rs1:
                     for r2 in rs2:
                         r1['col3'] = r2['col3']
                         yield r1
+            # left join
             elif rs1 and not rs2:
                 for r1 in rs1:
                     r1['col3'] = None
                     yield r1
-
+            # right join
             elif not rs1 and rs2:
                 for r2 in rs2:
                     r2['col2'] = None
                     yield r2
 
         conn = Conn(self.dbfile)
-        conn['t1'] = Inst(table1)
-        conn['t2'] = Inst(table2)
+        conn['t1'] = table1
+        conn['t2'] = table2
 
-        conn['t1_col3'] = conn.inst('t1', by='col1')\
-            .merge(add_col3, conn.inst('t2', 'col1'))
+        conn['t1_col3'] = conn['t1', 'col1']\
+            .merge(add_col3, conn['t2', 'col1'])
 
-        self.assertEqual(conn['t1_col3'], [
+        self.assertEqual(conn['t1_col3'].list(), [
             {'col1': 'a', 'col2': 4, 'col3': None},
             {'col1': 'a', 'col2': 5, 'col3': None},
             {'col1': 'b', 'col2': 1, 'col3': 1},
@@ -91,10 +95,19 @@ class TestGeneral(unittest.TestCase):
             {'col1': 'e', 'col2': None, 'col3': 4}]
         )
 
+        table2_1 = sorted(table2, key=lambda r: r['col1'])
+        conn['t1_col3_1'] = conn['t1', 'col1']\
+            .merge(add_col3, groupby(table2_1, lambda r: r['col1']))
+
+        self.assertEqual(conn['t1_col3'].list(), conn['t1_col3_1'].list())
+
     def test_concat(self):
         conn = Conn(self.dbfile)
-        conn['t1'] = Inst(table1)
-        conn['t1_double'] = Inst(table1).concat(conn.inst('t1'))
-        self.assertEqual(conn['t1'] + conn['t1'], conn['t1_double'])
-    
-        
+        conn['t1'] = table1
+        conn['t1_double'] = conn['t1'].concat(conn['t1'])
+        self.assertEqual(conn['t1'].list() + conn['t1'].list(),
+                         conn['t1_double'].list())
+
+        conn['t1_double'] = conn['t1'].concat(table1)
+        self.assertEqual(conn['t1'].list() + conn['t1'].list(),
+                         conn['t1_double'].list())
